@@ -2,9 +2,10 @@
 use strict;
 use warnings;
 package YAML::PP;
-use B;
 
-our $VERSION = '0.009_001'; # TRIAL VERSION
+our $VERSION = '0.010_001'; # TRIAL VERSION
+
+use YAML::PP::Schema;
 
 use base 'Exporter';
 our @EXPORT_OK = qw/ Load LoadFile Dump DumpFile /;
@@ -147,457 +148,6 @@ sub DumpFile {
     YAML::PP->new->dump_file($file, @data);
 }
 
-package YAML::PP::Schema;
-
-sub new {
-    my ($class, %args) = @_;
-
-    my $bool = delete $args{boolean};
-    $bool = 'perl' unless defined $bool;
-    if (keys %args) {
-        die "Unexpected arguments: " . join ', ', sort keys %args;
-    }
-    my $true;
-    my $false;
-    if ($bool eq 'JSON::PP') {
-        require JSON::PP;
-        $true = \&bool_jsonpp_true;
-        $false = \&bool_jsonpp_false;
-    }
-    elsif ($bool eq 'boolean') {
-        require boolean;
-        $true = \&bool_booleanpm_true;
-        $false = \&bool_booleanpm_false;
-    }
-    elsif ($bool eq 'perl') {
-        $true = \&bool_perl_true;
-        $false = \&bool_perl_false;
-    }
-    else {
-        die "Invalid value for 'boolean': '$bool'. Allowed: ('perl', 'boolean', 'JSON::PP')";
-    }
-
-    my $self = bless {
-        resolvers => {},
-        representers => {},
-        true => $true,
-        false => $false,
-    }, $class;
-    return $self;
-}
-
-sub resolvers { return $_[0]->{resolvers} }
-sub representers { return $_[0]->{representers} }
-sub true { return $_[0]->{true} }
-sub false { return $_[0]->{false} }
-
-sub load_subschemas {
-    my ($self, @schemas) = @_;
-    for my $s (@schemas) {
-        my $class = "YAML::PP::Schema::" . $s;
-        my $tags = $class->register(
-            schema => $self,
-        );
-    }
-}
-
-sub add_resolver {
-    my ($self, %args) = @_;
-    my $tag = $args{tag};
-    my $rule = $args{match};
-    my $resolvers = $self->resolvers;
-    my ($type, $match, $value) = @$rule;
-    my $implicit = $args{implicit};
-    $implicit = 1 unless defined $implicit;
-    my @resolvers;
-    if ($tag) {
-        my $res = $resolvers->{tag}->{ $tag } ||= {};
-        push @resolvers, $res;
-    }
-    if ($implicit) {
-        push @resolvers, $resolvers->{value} ||= {};
-    }
-    for my $res (@resolvers) {
-        if ($type eq 'equals') {
-            unless (exists $res->{equals}->{ $match }) {
-                $res->{equals}->{ $match } = $value;
-            }
-            next;
-        }
-        if ($type eq 'regex') {
-            push @{ $res->{regex} }, [ $match => $value ];
-        }
-    }
-}
-
-sub add_representer {
-    my ($self, %args) = @_;
-
-    my $representers = $self->representers;
-    if (my $flags = $args{flags}) {
-        my $rep = $representers->{flags} ||= [];
-        push @$rep, \%args;
-        return;
-    }
-    if (my $regex = $args{regex}) {
-        my $rep = $representers->{regex} ||= [];
-        push @$rep, \%args;
-        return;
-    }
-    if (defined(my $equals = $args{equals})) {
-        my $rep = $representers->{equals} ||= {};
-        $rep->{ $equals } = {
-            code => $args{code},
-        };
-        return;
-    }
-    if (my $undef = $args{undefined}) {
-        $representers->{undef} = $undef;
-        return;
-    }
-}
-
-sub load_scalar_tag {
-    my ($self, $event) = @_;
-    my $tag = $event->{tag};
-    my $value = $event->{value};
-    $value = '' unless defined $value;
-    my $resolvers = $self->resolvers;
-    my $res = $resolvers->{tag}->{ $tag };
-
-    if (my $equals = $res->{equals}) {
-        if (exists $equals->{ $value }) {
-            my $res = $equals->{ $value };
-            if (ref $res eq 'CODE') {
-                return $res->();
-            }
-            return $res;
-        }
-    }
-    if (my $regex = $res->{regex}) {
-        for my $item (@$regex) {
-            my ($re, $sub) = @$item;
-            my @matches = $value =~ $re;
-            if (@matches) {
-                return $sub->(@matches);
-            }
-        }
-        die "Tag $tag ($value)";
-    }
-#    die "Tag $tag ($value)";
-    return $value;
-}
-
-sub load_scalar {
-    my ($self, $style, $value) = @_;
-    if ($style ne ':') {
-        return $value;
-    }
-    my $resolvers = $self->resolvers;
-    my $res = $resolvers->{value};
-    $value = '' unless defined $value;
-
-    if (my $equals = $res->{equals}) {
-        if (exists $equals->{ $value }) {
-            my $res = $equals->{ $value };
-            if (ref $res eq 'CODE') {
-                return $res->();
-            }
-            return $res;
-        }
-    }
-    if (my $regex = $res->{regex}) {
-        for my $item (@$regex) {
-            my ($re, $sub) = @$item;
-            my @matches = $value =~ $re;
-            if (@matches) {
-                return $sub->(@matches);
-            }
-        }
-    }
-    return $value;
-}
-
-sub bool_jsonpp_true { JSON::PP::true() }
-
-sub bool_booleanpm_true { boolean::true() }
-
-sub bool_perl_true { 1 }
-
-sub bool_jsonpp_false { JSON::PP::false() }
-
-sub bool_booleanpm_false { boolean::false() }
-
-sub bool_perl_false { !1 }
-
-
-package YAML::PP::Schema::Base;
-
-sub register {
-    return {};
-}
-
-package YAML::PP::Schema::Failsafe;
-
-use base 'YAML::PP::Schema';
-
-sub register {
-    my ($self, %args) = @_;
-    my $schema = $args{schema};
-
-    $schema->add_resolver(
-        match => [ equals => '' => '' ],
-    );
-
-    $schema->add_representer(
-        undefined => sub {
-            my ($rep, $value) = @_;
-            return { quoted => "" };
-        },
-    );
-    return;
-}
-
-package YAML::PP::Schema::JSON;
-use base 'YAML::PP::Schema';
-
-my $RE_INT = qr{^(-?(?:0|[1-9][0-9]*))$};
-my $RE_FLOAT = qr{^(-?(?:0|[1-9][0-9]*)(?:\.[0-9]*)?(?:[eE][+-]?[0-9]+)?)$};
-
-sub new {
-    my ($class, %args) = @_;
-    my $self = bless {}, $class;
-    return $self;
-}
-
-sub _to_int { 0 + $_[0] }
-
-# DaTa++ && shmem++
-sub _to_float { unpack F => pack F => $_[0] }
-
-sub register {
-    my ($self, %args) = @_;
-    my $schema = $args{schema};
-
-    $schema->add_resolver(
-        tag => 'tag:yaml.org,2002:null',
-        match => [ equals => null => undef ],
-    );
-    $schema->add_resolver(
-        tag => 'tag:yaml.org,2002:null',
-        match => [ equals => '' => undef ],
-        implicit => 0,
-    );
-    $schema->add_resolver(
-        tag => 'tag:yaml.org,2002:bool',
-        match => [ equals => true => $schema->true ],
-    );
-    $schema->add_resolver(
-        tag => 'tag:yaml.org,2002:bool',
-        match => [ equals => false => $schema->false ],
-    );
-    $schema->add_resolver(
-        tag => 'tag:yaml.org,2002:int',
-        match => [ regex => $RE_INT => \&_to_int ],
-    );
-    $schema->add_resolver(
-        tag => 'tag:yaml.org,2002:float',
-        match => [ regex => $RE_FLOAT => \&_to_float ],
-    );
-    $schema->add_resolver(
-        tag => 'tag:yaml.org,2002:str',
-        match => [ regex => qr{^(.*)$} => sub { $_[0] } ],
-        implicit => 0,
-    );
-
-    $schema->add_representer(
-        undefined => sub {
-            my ($rep, $value) = @_;
-            return { plain => "null" };
-        },
-    );
-    $schema->add_representer(
-        reftype => "*",
-        code => sub {
-            die "Dumping references not supported yet";
-        },
-    );
-
-    my $int_flags = B::SVp_IOK;
-    my $float_flags = B::SVp_NOK;
-    $schema->add_representer(
-        flags => $int_flags,
-        code => sub {
-            my ($rep, $value) = @_;
-            if (int($value) ne $value) {
-                return { skip => 1 };
-            }
-            return { plain => "$value" };
-        },
-    );
-    my %special = ( (0+'nan').'' => '.nan', (0+'inf').'' => '.inf', (0-'inf').'' => '-.inf' );
-    $schema->add_representer(
-        flags => $float_flags,
-        code => sub {
-            my ($rep, $value) = @_;
-            # TODO is inf/nan supported in YAML JSON Schema?
-            if (exists $special{ $value }) {
-                return { plain => "$value" };
-            }
-            if (0.0 + $value ne $value) {
-                return { skip => 1 };
-            }
-            if (int($value) eq $value and not $value =~ m/\./) {
-                $value .= '.0';
-            }
-            return { plain => "$value" };
-        },
-    );
-    $schema->add_representer(
-        equals => $_,
-        code => sub {
-            my ($rep, $value) = @_;
-            return { quoted => "$value" };
-        },
-    ) for ("", qw/ true false null /);
-    $schema->add_representer(
-        regex => qr{$RE_INT|$RE_FLOAT},
-        code => sub {
-            my ($rep, $value) = @_;
-            return { quoted => "$value" };
-        },
-    );
-    return;
-}
-
-package YAML::PP::Schema::Core;
-use base 'YAML::PP::Schema';
-
-my $RE_INT_CORE = qr{^([+-]?(?:[0-9]+))$};
-my $RE_FLOAT_CORE = qr{^([+-]?(?:\.[0-9]+|[0-9]+(?:\.[0-9]*)?)(?:[eE][+-]?[0-9]+)?)$};
-my $RE_INT_OCTAL = qr{^0o([0-7]+)$};
-my $RE_INT_HEX = qr{^0x([0-9a-fA-F]+)$};
-
-
-sub new {
-    my ($class, %args) = @_;
-    my $self = bless {}, $class;
-    return $self;
-}
-
-sub _from_oct { oct $_[0] }
-sub _from_hex { hex $_[0] }
-
-sub register {
-    my ($self, %args) = @_;
-    my $schema = $args{schema};
-
-    $schema->add_resolver(
-        tag => 'tag:yaml.org,2002:null',
-        match => [ equals => $_ => undef ],
-    ) for (qw/ null NULL Null ~ /, '');
-    $schema->add_resolver(
-        tag => 'tag:yaml.org,2002:bool',
-        match => [ equals => $_ => $schema->true ],
-    ) for (qw/ true TRUE True /);
-    $schema->add_resolver(
-        tag => 'tag:yaml.org,2002:bool',
-        match => [ equals => $_ => $schema->false ],
-    ) for (qw/ false FALSE False /);
-    $schema->add_resolver(
-        tag => 'tag:yaml.org,2002:int',
-        match => [ regex => $RE_INT_CORE => \&YAML::PP::Schema::JSON::_to_int ],
-    );
-    $schema->add_resolver(
-        tag => 'tag:yaml.org,2002:int',
-        match => [ regex => $RE_INT_OCTAL => \&_from_oct ],
-    );
-    $schema->add_resolver(
-        tag => 'tag:yaml.org,2002:int',
-        match => [ regex => $RE_INT_HEX => \&_from_hex ],
-    );
-    $schema->add_resolver(
-        tag => 'tag:yaml.org,2002:float',
-        match => [ regex => $RE_FLOAT_CORE => \&YAML::PP::Schema::JSON::_to_float ],
-    );
-    $schema->add_resolver(
-        tag => 'tag:yaml.org,2002:float',
-        match => [ equals => $_ => 0 + "inf" ],
-    ) for (qw/ .inf .Inf .INF /);
-    $schema->add_resolver(
-        tag => 'tag:yaml.org,2002:float',
-        match => [ equals => $_ => 0 - "inf" ],
-    ) for (qw/ -.inf -.Inf -.INF /);
-    $schema->add_resolver(
-        tag => 'tag:yaml.org,2002:float',
-        match => [ equals => $_ => 0 + "nan" ],
-    ) for (qw/ .nan .NaN .NAN /);
-    $schema->add_resolver(
-        tag => 'tag:yaml.org,2002:str',
-        match => [ regex => qr{^(.*)$} => sub { $_[0] } ],
-        implicit => 0,
-    );
-
-    $schema->add_representer(
-        reftype => "*",
-        code => sub {
-            die "Dumping references not supported yet";
-        },
-    );
-
-    my $int_flags = B::SVp_IOK;
-    my $float_flags = B::SVp_NOK;
-    $schema->add_representer(
-        flags => $int_flags,
-        code => sub {
-            my ($rep, $value) = @_;
-            if (int($value) ne $value) {
-                return { skip => 1 };
-            }
-            return { plain => "$value" };
-        },
-    );
-    my %special = ( (0+'nan').'' => '.nan', (0+'inf').'' => '.inf', (0-'inf').'' => '-.inf' );
-    $schema->add_representer(
-        flags => $float_flags,
-        code => sub {
-            my ($rep, $value) = @_;
-            if (exists $special{ $value }) {
-                return { plain => $special{ $value } };
-            }
-            if (0.0 + $value ne $value) {
-                return { skip => 1 };
-            }
-            if (int($value) eq $value and not $value =~ m/\./) {
-                $value .= '.0';
-            }
-            return { plain => "$value" };
-        },
-    );
-    $schema->add_representer(
-        undefined => sub {
-            my ($rep, $value) = @_;
-            return { plain => "null" };
-        },
-    );
-    $schema->add_representer(
-        equals => $_,
-        code => sub {
-            my ($rep, $value) = @_;
-            return { quoted => "$value" };
-        },
-    ) for ("", qw/ true TRUE True false FALSE False null NULL Null ~ /);
-    $schema->add_representer(
-        regex => qr{$RE_INT_CORE|$RE_FLOAT_CORE|$RE_INT_OCTAL|$RE_INT_HEX},
-        code => sub {
-            my ($rep, $value) = @_;
-            return { quoted => "$value" };
-        },
-    );
-
-    return;
-}
-
 1;
 
 __END__
@@ -612,12 +162,30 @@ YAML::PP - YAML 1.2 processor
 
 =head1 SYNOPSIS
 
-WARNING: This is highly experimental.
+WARNING: This is not yet stable.
 
-Here are a few examples of what you can do right now:
+Here are a few examples of the basic load and dump methods:
 
     use YAML::PP;
     my $ypp = YAML::PP->new;
+    my $yaml = <<'EOM';
+    --- # Document one is a mapping
+    name: Tina
+    age: 29
+    favourite language: Perl
+
+    --- # Document two is a sequence
+    - plain string
+    - 'in single quotes'
+    - "in double quotes we have escapes! like \t and \n"
+    - | # a literal block scalar
+      line1
+      line2
+    - > # a folded block scalar
+      this is all one
+      single line because the
+      linebreaks will be folded
+    EOM
 
     my @documents = $ypp->load_string($yaml);
     my @documents = $ypp->load_file($filename);
@@ -640,6 +208,9 @@ Here are a few examples of what you can do right now:
     DumpFile($filename, @documents);
     DumpFile($filenhandle @documents);
 
+    my $ypp = YAML::PP->new(schema => [qw/ JSON Perl /]);
+    my $yaml = $yp->dump_string($data_with_perl_objects);
+
 
 Some utility scripts:
 
@@ -670,6 +241,29 @@ You can check out all current parse and load results from the
 yaml-test-suite here:
 L<https://perlpunk.github.io/YAML-PP-p5/test-suite.html>
 
+The process of loading and dumping is split into the following steps:
+
+    Load:
+
+    YAML Stream        Tokens        Event List        Data Structure
+              --------->    --------->        --------->
+                lex           parse           construct
+
+
+    Dump:
+
+    Data Structure       Event List        YAML Stream
+                --------->        --------->
+                represent           emit
+
+
+You can dump basic perl types like hashes, arrays, scalars (strings, numbers).
+For dumping blessed objects and things like coderefs have a look at
+L<YAML::PP::Schema::Perl>.
+
+For keeping your ordered L<Tie::IxHash> hashes, try out
+:<YAML::PP::Schema::Tie::IxHash>.
+
 =over
 
 =item L<YAML::PP::Lexer>
