@@ -2,7 +2,7 @@ use strict;
 use warnings;
 package YAML::PP::Lexer;
 
-our $VERSION = '0.009_001'; # TRIAL VERSION
+our $VERSION = '0.010_001'; # TRIAL VERSION
 
 use constant TRACE => $ENV{YAML_PP_TRACE} ? 1 : 0;
 use constant DEBUG => ($ENV{YAML_PP_DEBUG} || $ENV{YAML_PP_TRACE}) ? 1 : 0;
@@ -598,7 +598,9 @@ sub fetch_block {
         );
     }
     my $value = YAML::PP::Render->render_block_scalar($context, $chomp, \@lines);
+    my @eol = splice @tokens, -2;
     $self->push_subtokens( { name => 'BLOCK_SCALAR', value => $value }, \@tokens );
+    $self->push_tokens([ @eol ]);
     return 0;
 }
 
