@@ -3,11 +3,17 @@ use strict;
 use warnings;
 package YAML::PP::Parser;
 
-our $VERSION = '0.009_001'; # TRIAL VERSION
+our $VERSION = '0.010_001'; # TRIAL VERSION
 
 use constant TRACE => $ENV{YAML_PP_TRACE} ? 1 : 0;
 use constant DEBUG => ($ENV{YAML_PP_DEBUG} || $ENV{YAML_PP_TRACE}) ? 1 : 0;
 
+use YAML::PP::Common qw/
+    YAML_PLAIN_SCALAR_STYLE YAML_SINGLE_QUOTED_SCALAR_STYLE
+    YAML_DOUBLE_QUOTED_SCALAR_STYLE
+    YAML_LITERAL_SCALAR_STYLE YAML_FOLDED_SCALAR_STYLE
+    YAML_FLOW_SEQUENCE_STYLE YAML_FLOW_MAPPING_STYLE
+/;
 use YAML::PP::Render;
 use YAML::PP::Lexer;
 use YAML::PP::Grammar qw/ $GRAMMAR /;
@@ -92,13 +98,13 @@ sub init {
 
 sub parse_string {
     my ($self, $yaml) = @_;
-    $self->reader->set_input( $yaml );
+    $self->set_reader(YAML::PP::Reader->new( input => $yaml ));
     $self->parse();
 }
 
 sub parse_file {
     my ($self, $file) = @_;
-    $self->reader->set_input( $file );
+    $self->set_reader(YAML::PP::Reader::File->new( input => $file ));
     $self->parse();
 }
 
@@ -249,6 +255,7 @@ my %nodetypes = (
     FLOWMAP => 'NODETYPE_FLOWMAP',
     FLOWMAPVALUE => 'NODETYPE_FLOWMAPVALUE',
     FLOWSEQ => 'NODETYPE_FLOWSEQ',
+    FLOWSEQ_NEXT => 'FLOWSEQ_NEXT',
 );
 
 sub parse_document {
@@ -259,14 +266,16 @@ sub parse_document {
     my $next_tokens = $lexer->next_tokens;
     my $event_types = $self->events;
     my $stack = $self->event_stack;
-    $lexer->fetch_next_tokens(0);
     LINE: while (1) {
 
         TRACE and $self->info("----------------------- LOOP");
         TRACE and $self->debug_events;
 
         unless (@$next_tokens) {
-            return $self->end_document;
+            $lexer->fetch_next_tokens(0);
+            unless (@$next_tokens) {
+                return $self->end_document;
+            }
         }
         if ( $next_tokens->[0]->{name} eq 'EOL' ) {
             push @{ $self->tokens }, shift @$next_tokens;
@@ -280,20 +289,15 @@ sub parse_document {
         }
 
         DEBUG and $self->info("----------------> parse_next_line");
-        while (1) {
-            unless ($self->new_node) {
-                my $new_rule = $nodetypes{ $event_types->[-1] }
-                    or die "Expected document end";
-                $self->set_rule( $new_rule );
-            }
 
-            my $res = $self->parse_tokens();
+        unless ($self->new_node) {
+            my $new_rule = $nodetypes{ $event_types->[-1] }
+                or die "Expected document end";
+            $self->set_rule( $new_rule );
+        }
+
+        my $res = $self->parse_tokens();
 
-            last if (not @$next_tokens or $next_tokens->[0]->{column} == 0);
-        }
-        unless (@$next_tokens) {
-            $lexer->fetch_next_tokens(0);
-        }
     }
 
     return;
@@ -357,7 +361,7 @@ sub check_indent {
             return;
         }
         else {
-            $self->scalar_event({ style => ':', value => undef });
+            $self->scalar_event({ style => YAML_PLAIN_SCALAR_STYLE, value => undef });
         }
     }
 
@@ -384,7 +388,7 @@ sub end_document {
         die "Unexpected end of flow context";
     }
     if ($self->new_node) {
-        $self->scalar_event({ style => ':', value => undef });
+        $self->scalar_event({ style => YAML_PLAIN_SCALAR_STYLE, value => undef });
     }
     $self->remove_nodes(-1);
 }
@@ -396,7 +400,8 @@ my %next_event = (
     SEQ0 => 'SEQ0',
     DOC => 'DOC',
     STR => 'STR',
-    FLOWSEQ => 'FLOWSEQ',
+    FLOWSEQ => 'FLOWSEQ_NEXT',
+    FLOWSEQ_NEXT => 'FLOWSEQ',
     FLOWMAP => 'FLOWMAPVALUE',
     FLOWMAPVALUE => 'FLOWMAP',
 );
@@ -434,20 +439,20 @@ my %event_to_method = (
 #            $properties = $info;
 #        }
 #        elsif ($type eq 'scalar') {
-#            $info->{event_name} = 'scalar_event';
+#            $info->{name} = 'scalar_event';
 #            $event_types->[-1] = $next_event{ $event_types->[-1] };
 #            push @send_events, $info;
 #        }
 #        elsif ($type eq 'begin') {
 #            my $name = $info->{name};
-#            $info->{event_name} = $event_to_method{ $name } . '_start_event';
+#            $info->{name} = $event_to_method{ $name } . '_start_event';
 #            push @{ $event_types }, $name;
 #            push @{ $self->offset }, $info->{offset};
 #            push @send_events, $info;
 #        }
 #        elsif ($type eq 'end') {
 #            my $name = $info->{name};
-#            $info->{event_name} = $event_to_method{ $name } . '_end_event';
+#            $info->{name} = $event_to_method{ $name } . '_end_event';
 #            $self->$type($name, $info);
 #            push @send_events, $info;
 #            if (@$event_types) {
@@ -458,15 +463,15 @@ my %event_to_method = (
 #            if ($properties) {
 #                $self->exception("Parse error: Alias not allowed in this context");
 #            }
-#            $info->{event_name} = 'alias_event';
+#            $info->{name} = 'alias_event';
 #            $event_types->[-1] = $next_event{ $event_types->[-1] };
 #            push @send_events, $info;
 #        }
 #    }
 #    @$event_stack = ();
 #    for my $info (@send_events) {
-#        DEBUG and $self->debug_event( $info->{event_name} => $info );
-#        $self->callback->($self, $info->{event_name}, $info);
+#        DEBUG and $self->debug_event( $info );
+#        $self->callback->($self, $info->{name}, $info);
 #    }
 #}
 
@@ -490,6 +495,8 @@ sub parse_tokens {
     TRACE and $self->debug_yaml;
     DEBUG and $self->debug_next_line;
 
+    my $event_types = $self->events;
+    my $offsets = $self->offset;
     my $tokens = $self->tokens;
     my $next_tokens = $self->lexer->next_tokens;
     $res->{offset} = $next_tokens->[0]->{column};
@@ -504,7 +511,7 @@ sub parse_tokens {
         my $got = $next_tokens->[0]->{name};
         if ($got eq 'CONTEXT') {
             my $context = shift @$next_tokens;
-            my $indent = $self->offset->[-1];
+            my $indent = $offsets->[-1];
             $indent++ unless $self->lexer->flowcontext;
             my $method = $fetch_method{ $context->{value} };
             my $partial = $self->lexer->$method($indent, $context->{value});
@@ -533,22 +540,31 @@ sub parse_tokens {
         if ($new) {
             DEBUG and $self->got("NEW: $new");
 
-            if ($def->{return}) {
-                $self->set_rule($new);
-                return $res;
-            }
             $next_rule_name = $new;
+            $self->set_rule($next_rule_name);
+
+            if ($def->{return}) {
+                return 1;
+            }
 
-            $next_rule = $GRAMMAR->{ $next_rule_name }
-                or die "Unexpected rule $next_rule_name";
-            next RULE;
         }
         elsif ($def->{return}) {
-            $self->set_new_node(0);
-            return $res;
+            if ($got eq 'EOL' or not @$next_tokens) {
+                return 1;
+            }
+
+            $next_rule_name = $nodetypes{ $event_types->[-1] }
+                or die "Expected document end";
+            $self->set_rule($next_rule_name);
+
         }
-        $next_rule_name .= " - $got"; # for debugging
-        $next_rule = $def;
+        else {
+            $next_rule_name .= " - $got"; # for debugging
+            $next_rule = $def;
+            next RULE;
+        }
+        $next_rule = $GRAMMAR->{ $next_rule_name }
+            or die "Unexpected rule $next_rule_name";
 
     }
 
@@ -560,8 +576,8 @@ sub end_sequence {
     my $event_types = $self->events;
     pop @{ $event_types };
     pop @{ $self->offset };
-    my $info = { event_name => 'sequence_end_event' };
-    $self->callback->($self, $info->{event_name} => $info );
+    my $info = { name => 'sequence_end_event' };
+    $self->callback->($self, $info->{name} => $info );
     $event_types->[-1] = $next_event{ $event_types->[-1] };
 }
 
@@ -576,14 +592,14 @@ sub remove_nodes {
             last;
         }
         if ($exp eq 'MAPVALUE') {
-            $self->scalar_event({ style => ':', value => undef });
+            $self->scalar_event({ style => YAML_PLAIN_SCALAR_STYLE, value => undef });
             $exp = 'MAP';
         }
         my $info = { name => $exp };
-        $info->{event_name} = $event_to_method{ $exp } . '_end_event';
+        $info->{name} = $event_to_method{ $exp } . '_end_event';
         pop @{ $event_types };
         pop @{ $offset };
-        $self->callback->($self, $info->{event_name} => $info );
+        $self->callback->($self, $info->{name} => $info );
         $event_types->[-1] = $next_event{ $event_types->[-1] };
         $exp = $event_types->[-1];
     }
@@ -606,14 +622,19 @@ sub start_stream {
     my ($self) = @_;
     push @{ $self->events }, 'STR';
     push @{ $self->offset }, -1;
-    $self->callback->($self, 'stream_start_event', {});
+    $self->callback->($self, 'stream_start_event', {
+        name => 'stream_start_event',
+    });
 }
 
 sub start_document {
     my ($self, $implicit) = @_;
     push @{ $self->events }, 'DOC';
     push @{ $self->offset }, -1;
-    $self->callback->($self, 'document_start_event', { implicit => $implicit });
+    $self->callback->($self, 'document_start_event', {
+        name => 'document_start_event',
+        implicit => $implicit,
+    });
 }
 
 sub start_sequence {
@@ -627,7 +648,7 @@ sub start_sequence {
     }
     push @{ $offsets }, $offset;
     my $event_stack = $self->event_stack;
-    my $info = {};
+    my $info = { name => 'sequence_start_event' };
     if (@$event_stack and $event_stack->[-1]->[0] eq 'properties') {
         my $properties = pop @$event_stack;
         $self->node_properties($properties->[1], $info);
@@ -650,7 +671,7 @@ sub start_flow_sequence {
     push @{ $offsets }, $new_offset;
 
     my $event_stack = $self->event_stack;
-    my $info = { style => 'flow' };
+    my $info = { style => YAML_FLOW_SEQUENCE_STYLE, name => 'sequence_start_event'  };
     if (@$event_stack and $event_stack->[-1]->[0] eq 'properties') {
         $self->fetch_inline_properties($event_stack, $info);
     }
@@ -672,7 +693,7 @@ sub start_flow_mapping {
     push @{ $offsets }, $new_offset;
 
     my $event_stack = $self->event_stack;
-    my $info = { style => 'flow' };
+    my $info = { name => 'mapping_start_event', style => YAML_FLOW_MAPPING_STYLE };
     if (@$event_stack and $event_stack->[-1]->[0] eq 'properties') {
         $self->fetch_inline_properties($event_stack, $info);
     }
@@ -684,15 +705,9 @@ sub end_flow_sequence {
     my $event_types = $self->events;
     pop @{ $event_types };
     pop @{ $self->offset };
-    my $info = { event_name => 'sequence_end_event' };
-    $self->callback->($self, $info->{event_name}, $info);
+    my $info = { name => 'sequence_end_event' };
+    $self->callback->($self, $info->{name}, $info);
     $event_types->[-1] = $next_event{ $event_types->[-1] };
-    if ($event_types->[-1] =~ m/^FLOW/) {
-        $self->set_new_node(1);
-    }
-    else {
-        $self->set_new_node(0);
-    }
 }
 
 sub end_flow_mapping {
@@ -700,10 +715,9 @@ sub end_flow_mapping {
     my $event_types = $self->events;
     pop @{ $event_types };
     pop @{ $self->offset };
-    my $info = { event_name => 'mapping_end_event' };
-    $self->callback->($self, $info->{event_name}, $info);
+    my $info = { name => 'mapping_end_event' };
+    $self->callback->($self, $info->{name}, $info);
     $event_types->[-1] = $next_event{ $event_types->[-1] };
-    # TODO
 }
 
 sub start_mapping {
@@ -712,7 +726,7 @@ sub start_mapping {
     push @{ $self->events }, 'MAP';
     push @{ $offsets }, $offset;
     my $event_stack = $self->event_stack;
-    my $info = {};
+    my $info = { name => 'mapping_start_event' };
     if (@$event_stack and $event_stack->[-1]->[0] eq 'properties') {
         my $properties = pop @$event_stack;
         $self->node_properties($properties->[1], $info);
@@ -726,7 +740,10 @@ sub end_doc {
     $self->exception("Unexpected event type $last") unless $last eq 'DOC';
     pop @{ $self->offset };
     $self->set_tagmap({ '!!' => "tag:yaml.org,2002:" });
-    $self->callback->($self, 'document_end_event', { implicit => $implicit });
+    $self->callback->($self, 'document_end_event', {
+        name => 'document_end_event',
+        implicit => $implicit,
+    });
 }
 
 sub end_stream {
@@ -734,7 +751,9 @@ sub end_stream {
     my $last = pop @{ $self->events };
     $self->exception("Unexpected event type $last") unless $last eq 'STR';
     pop @{ $self->offset };
-    $self->callback->($self, 'stream_end_event', { });
+    $self->callback->($self, 'stream_end_event', {
+        name => 'stream_end_event',
+    });
 }
 
 sub fetch_inline_properties {
@@ -787,9 +806,9 @@ sub scalar_event {
         $properties = $self->node_properties($properties->[1], $info);
     }
 
+    $info->{name} = 'scalar_event';
     $self->callback->($self, 'scalar_event', $info);
     if ($event_types->[-1] =~ m/^FLOW/) {
-        $self->set_new_node(1);
     }
     else {
         $self->set_new_node(0);
@@ -804,9 +823,9 @@ sub alias_event {
         $self->exception("Parse error: Alias not allowed in this context");
     }
     my $event_types = $self->events;
+    $info->{name} = 'alias_event';
     $self->callback->($self, 'alias_event', $info);
     if ($event_types->[-1] =~ m/^FLOW/) {
-        $self->set_new_node(1);
     }
     else {
         $self->set_new_node(0);
@@ -853,78 +872,10 @@ sub _remaining_tokens {
 
 sub event_to_test_suite {
     my ($self, $event) = @_;
-    if (ref $event) {
-        my ($ev, $info) = @$event;
-        if ($event_to_method{ $ev }) {
-            $ev = $event_to_method{ $ev } . "_event";
-        }
-        my $string;
-        my $content = $info->{value};
-
-        my $properties = '';
-        $properties .= " &$info->{anchor}" if defined $info->{anchor};
-        $properties .= " <$info->{tag}>" if defined $info->{tag};
-
-        if ($ev eq 'document_start_event') {
-            $string = "+DOC";
-            $string .= " ---" unless $info->{implicit};
-        }
-        elsif ($ev eq 'document_end_event') {
-            $string = "-DOC";
-            $string .= " ..." unless $info->{implicit};
-        }
-        elsif ($ev eq 'stream_start_event') {
-            $string = "+STR";
-        }
-        elsif ($ev eq 'stream_end_event') {
-            $string = "-STR";
-        }
-        elsif ($ev eq 'mapping_start_event') {
-            $string = "+MAP";
-            $string .= $properties;
-            if (0) {
-                # doesn't match yaml-test-suite format
-                if ($info->{style} and $info->{style} eq 'flow') {
-                    $string .= " {}";
-                }
-            }
-        }
-        elsif ($ev eq 'sequence_start_event') {
-            $string = "+SEQ";
-            $string .= $properties;
-            if (0) {
-                # doesn't match yaml-test-suite format
-                if ($info->{style} and $info->{style} eq 'flow') {
-                    $string .= " []";
-                }
-            }
-        }
-        elsif ($ev eq 'mapping_end_event') {
-            $string = "-MAP";
-        }
-        elsif ($ev eq 'sequence_end_event') {
-            $string = "-SEQ";
-        }
-        elsif ($ev eq 'scalar_event') {
-            $string = '=VAL';
-            $string .= $properties;
-            if (defined $content) {
-                $content =~ s/\\/\\\\/g;
-                $content =~ s/\t/\\t/g;
-                $content =~ s/\r/\\r/g;
-                $content =~ s/\n/\\n/g;
-                $content =~ s/[\b]/\\b/g;
-            }
-            else {
-                $content = '';
-            }
-            $string .= ' ' . $info->{style} . $content;
-        }
-        elsif ($ev eq 'alias_event') {
-            $string = "=ALI *$content";
-        }
-        return $string;
+    if (ref $event eq 'ARRAY') {
+        return YAML::PP::Common::event_to_test_suite($event->[1]);
     }
+    return YAML::PP::Common::event_to_test_suite($event);
 }
 
 sub debug_events {
@@ -986,8 +937,8 @@ sub _colorize_warn {
 }
 
 sub debug_event {
-    my ($self, $event, $info) = @_;
-    my $str = $self->event_to_test_suite([$event, $info]);
+    my ($self, $event) = @_;
+    my $str = YAML::PP::Common::event_to_test_suite($event);
     require Term::ANSIColor;
     warn Term::ANSIColor::colored(["magenta"], "============ $str"), "\n";
 }
@@ -1135,7 +1086,7 @@ sub cb_mapkey {
     my ($self, $token) = @_;
     my $stack = $self->event_stack;
     my $info = {
-        style => ':',
+        style => YAML_PLAIN_SCALAR_STYLE,
         value => $token->{value},
         offset => $token->{column},
     };
@@ -1156,13 +1107,14 @@ sub cb_send_scalar {
     my ($self, $res) = @_;
     my $last = pop @{ $self->event_stack };
     $self->scalar_event($last->[1]);
+    $self->set_new_node(0);
 }
 
 sub cb_empty_mapkey {
     my ($self, $token) = @_;
     my $stack = $self->event_stack;
     my $info = {
-        style => ':',
+        style => YAML_PLAIN_SCALAR_STYLE,
         value => undef,
         offset => $token->{column},
     };
@@ -1183,14 +1135,14 @@ sub cb_send_alias {
     my ($self, $token) = @_;
     my $alias = substr($token->{value}, 1);
     $self->alias_event({ value => $alias });
-    $self->set_new_node(1);
+    $self->set_new_node(0);
 }
 
 sub cb_send_alias_from_stack {
     my ($self, $token) = @_;
     my $last = pop @{ $self->event_stack };
     $self->alias_event($last->[1]);
-    $self->set_new_node(1);
+    $self->set_new_node(0);
 }
 
 sub cb_alias {
@@ -1209,12 +1161,11 @@ sub cb_question {
 
 sub cb_flow_question {
     my ($self, $res) = @_;
-    $self->set_new_node(1);
 }
 
 sub cb_empty_complexvalue {
     my ($self, $res) = @_;
-    $self->scalar_event({ style => ':', value => undef });
+    $self->scalar_event({ style => YAML_PLAIN_SCALAR_STYLE, value => undef });
 }
 
 sub cb_questionstart {
@@ -1244,7 +1195,9 @@ sub cb_take_quoted {
     my $subtokens = $token->{subtokens};
     my $stack = $self->event_stack;
     my $info = {
-        style => $subtokens->[0]->{value},
+        style => $subtokens->[0]->{value} eq '"'
+            ? YAML_DOUBLE_QUOTED_SCALAR_STYLE
+            : YAML_SINGLE_QUOTED_SCALAR_STYLE,
         value => $token->{value},
         offset => $token->{column},
     };
@@ -1259,7 +1212,9 @@ sub cb_quoted_multiline {
     my $subtokens = $token->{subtokens};
     my $stack = $self->event_stack;
     my $info = {
-        style => $subtokens->[0]->{value},
+        style => $subtokens->[0]->{value} eq '"'
+            ? YAML_DOUBLE_QUOTED_SCALAR_STYLE
+            : YAML_SINGLE_QUOTED_SCALAR_STYLE,
         value => $token->{value},
         offset => $token->{column},
     };
@@ -1280,7 +1235,7 @@ sub cb_send_plain_multi {
     my ($self, $token) = @_;
     my $stack = $self->event_stack;
     my $info = {
-        style => ':',
+        style => YAML_PLAIN_SCALAR_STYLE,
         value => $token->{value},
         offset => $token->{column},
     };
@@ -1295,7 +1250,7 @@ sub cb_start_plain {
     my ($self, $token) = @_;
     my $stack = $self->event_stack;
     my $info = {
-            style => ':',
+            style => YAML_PLAIN_SCALAR_STYLE,
             value => $token->{value},
             offset => $token->{column},
     };
@@ -1308,23 +1263,26 @@ sub cb_start_plain {
 sub cb_start_flowseq {
     my ($self, $token) = @_;
     $self->start_flow_sequence($token->{column});
-    $self->set_new_node(0);
 }
 
 sub cb_start_flowmap {
     my ($self, $token) = @_;
     $self->start_flow_mapping($token->{column});
-    $self->set_new_node(0);
 }
 
 sub cb_end_flowseq {
     my ($self, $res) = @_;
     $self->end_flow_sequence;
+    $self->set_new_node(0);
 }
 
 sub cb_flow_comma {
     my ($self) = @_;
+    my $event_types = $self->events;
     $self->set_new_node(0);
+    if ($event_types->[-1] =~ m/^FLOWSEQ/) {
+        $event_types->[-1] = $next_event{ $event_types->[-1] };
+    }
 }
 
 sub cb_flow_colon {
@@ -1336,7 +1294,7 @@ sub cb_empty_flow_mapkey {
     my ($self, $token) = @_;
     my $stack = $self->event_stack;
     my $info = {
-        style => ':',
+        style => YAML_PLAIN_SCALAR_STYLE,
         value => undef,
         offset => $token->{column},
     };
@@ -1344,7 +1302,7 @@ sub cb_empty_flow_mapkey {
         $self->fetch_inline_properties($stack, $info);
     }
     $self->scalar_event($info);
-    $self->set_new_node(1);
+    $self->set_new_node(0);
 }
 
 sub cb_end_flowmap {
@@ -1364,7 +1322,7 @@ sub cb_flow_plain {
     my ($self, $token) = @_;
     my $stack = $self->event_stack;
     my $info = {
-        style => ':',
+        style => YAML_PLAIN_SCALAR_STYLE,
         value => $token->{value},
         offset => $token->{column},
     };
@@ -1378,7 +1336,7 @@ sub cb_flowkey_plain {
     my ($self, $token) = @_;
     my $stack = $self->event_stack;
     my $info = {
-        style => ':',
+        style => YAML_PLAIN_SCALAR_STYLE,
         value => $token->{value},
         offset => $token->{column},
     };
@@ -1386,38 +1344,41 @@ sub cb_flowkey_plain {
         $self->fetch_inline_properties($stack, $info);
     }
     $self->scalar_event($info);
-}
-
-sub cb_flowkey_quoted {
-    my ($self, $token) = @_;
-    my $stack = $self->event_stack;
-    my $subtokens = $token->{subtokens};
-    my $info = {
-        style => $subtokens->[0]->{value},
-        value => $token->{value},
-        offset => $token->{column},
-    };
-    if (@$stack and $stack->[-1]->[0] eq 'properties') {
-        $self->fetch_inline_properties($stack, $info);
-    }
-    $self->scalar_event($info);
-}
-
-sub cb_empty_flowmap_value {
-    my ($self, $token) = @_;
-    my $stack = $self->event_stack;
-    my $info = {
-        style => ':',
-        value => undef,
-        offset => $token->{column},
-    };
-    if (@$stack and $stack->[-1]->[0] eq 'properties') {
-        $self->fetch_inline_properties($stack, $info);
-    }
-    $self->scalar_event($info);
     $self->set_new_node(0);
 }
 
+sub cb_flowkey_quoted {
+    my ($self, $token) = @_;
+    my $stack = $self->event_stack;
+    my $subtokens = $token->{subtokens};
+    my $info = {
+        style => $subtokens->[0]->{value} eq '"'
+            ? YAML_DOUBLE_QUOTED_SCALAR_STYLE
+            : YAML_SINGLE_QUOTED_SCALAR_STYLE,
+        value => $token->{value},
+        offset => $token->{column},
+    };
+    if (@$stack and $stack->[-1]->[0] eq 'properties') {
+        $self->fetch_inline_properties($stack, $info);
+    }
+    $self->scalar_event($info);
+    $self->set_new_node(0);
+}
+
+sub cb_empty_flowmap_value {
+    my ($self, $token) = @_;
+    my $stack = $self->event_stack;
+    my $info = {
+        style => YAML_PLAIN_SCALAR_STYLE,
+        value => undef,
+        offset => $token->{column},
+    };
+    if (@$stack and $stack->[-1]->[0] eq 'properties') {
+        $self->fetch_inline_properties($stack, $info);
+    }
+    $self->scalar_event($info);
+}
+
 sub cb_insert_map_alias {
     my ($self, $res) = @_;
     my $stack = $self->event_stack;
@@ -1442,7 +1403,7 @@ sub cb_insert_empty_map {
     my ($self, $token) = @_;
     my $stack = $self->event_stack;
     my $info = {
-        style => ':',
+        style => YAML_PLAIN_SCALAR_STYLE,
         value => undef,
         offset => $token->{column},
     };
@@ -1459,7 +1420,9 @@ sub cb_send_block_scalar {
     my $type = $token->{subtokens}->[0]->{value};
     my $stack = $self->event_stack;
     my $info = {
-        style => $type,
+        style => $type eq '|'
+            ? YAML_LITERAL_SCALAR_STYLE
+            : YAML_FOLDED_SCALAR_STYLE,
         value => $token->{value},
         offset => $token->{column},
     };
